/**
 * initApp handles setting up UI event listeners and registering Firebase auth listeners:
 *  - firebase.auth().onAuthStateChanged: This listener is called when the user is signed in or
 *    out, and that is where we update the UI.
 */

let email;
let uid;
let restName;
let traderName;
let restId;
let traderId;

var bs_modal = $("#modal");
var image = document.getElementById("image");
var cropper, reader, file;
let downloadUrl = " ";

var maxwidth = 512;
var maxheight = 512;

var finalAspectRatio = 1;

var imgwidth = 0;
var imgheight = 0;

$("body").on("change", ".image", function (e) {
  var files = e.target.files;
  var done = function (url) {
    image.src = url;
    image.onload = function () {
    //   imgwidth = this.width;
    //   imgheight = this.height;
    //   if (imgwidth === maxwidth && imgheight === maxheight) {
        console.log("Image size is " + maxwidth + "X" + maxheight);
        bs_modal.modal("show");
    //   } else {
    //     Swal.fire({
    //       icon: "error",
    //       title: "Oops...",
    //       text: "Image size must be " + maxwidth + "w by " + maxheight + "h",
    //     });
    //   }
    };
  };

  if (files && files.length > 0) {
    file = files[0];

    if (URL) {
      done(URL.createObjectURL(file));
    } else if (FileReader) {
      reader = new FileReader();
      reader.onload = function (e) {
        done(reader.result);
      };
      reader.readAsDataURL(file);
    }
  }
});

bs_modal
  .on("shown.bs.modal", function () {
    cropper = new Cropper(image, {
      aspectRatio: finalAspectRatio,
      viewMode: 1,
      preview: ".preview",
    });
  })
  .on("hidden.bs.modal", function () {
    cropper.destroy();
    cropper = null;
  });

$("#crop").click(function () {
  document.getElementById("crop").disabled = true;

  canvas = cropper.getCroppedCanvas({
    imageSmoothingEnabled: true,
    imageSmoothingQuality: "high",
  });

  canvas.toBlob(function (blob) {
    url = URL.createObjectURL(blob);
    let imPreview = document.getElementById("imgPreview");

    var reader = new FileReader();
    reader.readAsDataURL(blob);
    reader.onloadend = function () {
      var base64data = reader.result;
      console.log(base64data);
      // Get a reference to the storage service, which is used to create references in your storage bucket
      const storage = firebase.storage();

      // Create a storage reference from our storage service
      const storageRef = storage.ref();
      storageRef
        .child("branches/" + generateUUID())
        .putString(base64data, "data_url")
        .then((snapshot) => {
          snapshot.ref
            .getDownloadURL()
            .then(function (downloadURL) {
              console.log("File available at", downloadURL);
              downloadUrl = downloadURL;
              imPreview.src = downloadURL;
              document.getElementById("crop").disabled = false;

              bs_modal.modal("hide");
            })
            .catch(function (error) {
              document.getElementById("crop").disabled = false;

              console.log("Error getting document:", error);
            });
          console.log("Uploaded a data_url string!");
        });
    };
  });
});

function initMap() {
  const center = { lat: 50.064192, lng: -130.605469 };
  // Create a bounding box with sides ~10km away from the center point
  const defaultBounds = {
    north: center.lat + 0.1,
    south: center.lat - 0.1,
    east: center.lng + 0.1,
    west: center.lng - 0.1,
  };
  const input = document.getElementById("storeAddress");
  const options = {
    bounds: defaultBounds,

    fields: ["geometry", "name", "formatted_address"],
    origin: center,
    strictBounds: false,
    types: ["establishment"],
  };
  autocomplete = new google.maps.places.Autocomplete(input, options);
  autocomplete.addListener("place_changed", fillInAddress);
  var mapProp = {
    center: new google.maps.LatLng(-24.653257, 25.906792),
    zoom: 10,
    mapTypeId: google.maps.MapTypeId.ROADMAP,
  };
  var map = new google.maps.Map(document.getElementById("googleMap"), mapProp);

  google.maps.event.addListener(map, "click", function (event) {
    var result = [event.latLng.lat(), event.latLng.lng()];
    transition(result);
  });
}

var numDeltas = 100;
var delay = 10; //milliseconds
var i = 0;
var deltaLat;
var deltaLng;

function transition(result) {
  i = 0;
  deltaLat = (result[0] - position[0]) / numDeltas;
  deltaLng = (result[1] - position[1]) / numDeltas;
  moveMarker();
}

function moveMarker() {
  position[0] += deltaLat;
  position[1] += deltaLng;
  var latlng = new google.maps.LatLng(position[0], position[1]);
  marker.setTitle("Latitude:" + position[0] + " | Longitude:" + position[1]);
  marker.setPosition(latlng);
  if (i != numDeltas) {
    i++;
    setTimeout(moveMarker, delay);
  }
}

function initApp() {
  // Listening for auth state changes.
  // [START authstatelistener]
  firebase.auth().onAuthStateChanged(function (user) {
    if (user) {
      // User is signed in.

      email = user.email;
      let emailVerified = user.emailVerified;
      uid = user.uid;
      let providerData = user.providerData;

      console.log(user.metadata.lastSignInTime);
      let db = firebase.firestore();

      let docRef = db.collection("traders").doc(uid);

      docRef
        .get()
        .then(function (doc) {
          if (doc.exists) {
            restName = doc.data().restaurant;
            restId = doc.data().restId;
            traderId = doc.data().traderId;
            traderName = doc.data().traderFullName;
            document.getElementById("proName").innerText =
              traderName + " (" + restName + ")";
          } else {
            Swal.fire({
              icon: "error",
              title: "Oops...",
              text: "You do not have access to this platform.",
            });
            toggleSignOut();
            // doc.data() will be undefined in this case
            console.log("No such document!");
          }
        })
        .catch(function (error) {
          console.log("Error getting document:", error);
        });
    } else {
      // User is signed out.
      console.log("User is signed out");
      window.location.href = "index.html";
    }
  });
  // [END authstatelistener]
}

window.onload = function () {
  initApp();
};

function toggleSignOut() {
  // [START signout]
  firebase.auth().signOut();
  // [END signout]
}

$(document).on("keydown", function (e) {
  var keyCode = e.which || e.keyCode;
  if (keyCode == 13) {
    // enter key code
    toggleSignIn();
  }
});

$(document).ready(function () {
  $("#cuisineType").select2();
});

let placeSearch;
let autocomplete;
let lat;
let lng;
let branchAddress;
let db = firebase.firestore();
function fillInAddress() {
  // Get the place details from the autocomplete object.
  const place = autocomplete.getPlace();
  console.log(place);

  lat = place.geometry.location.lat();
  // get lng
  lng = place.geometry.location.lng();
  branchAddress = place.name + ", " + place.formatted_address;

  console.log(lat + " " + lng + " " + branchAddress);

  var mapProp = {
    center: new google.maps.LatLng(lat, lng),
    zoom: 18,
    mapTypeId: google.maps.MapTypeId.ROADMAP,
  };
  var map = new google.maps.Map(document.getElementById("googleMap"), mapProp);
  var marker = new google.maps.Marker({
    position: new google.maps.LatLng(lat, lng),
  });

  marker.setMap(map);
}

function generateUUID() {
  // Public Domain/MIT
  var d = new Date().getTime(); //Timestamp
  var d2 = (performance && performance.now && performance.now() * 1000) || 0; //Time in microseconds since page-load or 0 if unsupported
  return "xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx".replace(/[xy]/g, function (c) {
    var r = Math.random() * 16; //random number between 0 and 16
    if (d > 0) {
      //Use timestamp until depleted
      r = (d + r) % 16 | 0;
      d = Math.floor(d / 16);
    } else {
      //Use microseconds since page-load if supported
      r = (d2 + r) % 16 | 0;
      d2 = Math.floor(d2 / 16);
    }
    return (c === "x" ? r : (r & 0x3) | 0x8).toString(16);
  });
}
let stFloor = " ";
function addRest() {
  let cuisineTypeIds = $("#cuisineType").select2("val");
  let cuisineType = $("#cuisineType").select2("data");
  let cuisineTypeArray = [];
  for (let j = 0; j < cuisineType.length; j++) {
    cuisineTypeArray.push(cuisineType[j].text);
  }

  let stName = document.getElementById("stName").value;
  let stOpenTime = document.getElementById("stOpenTime").value;
  let stCloseTime = document.getElementById("stCloseTime").value;

  stFloor = document.getElementById("stFloor").value;
  let stPlot = document.getElementById("stPlot").value;

  // let status = document.getElementById('active').value;
  let stFName = document.getElementById("stFName").value;
  let stLName = document.getElementById("stLName").value;

  let stEmail = document.getElementById("stEmail").value;

  let stPhone = document.getElementById("stPhone").value;

  // stBaking = document.getElementById('stBaking').value;
  // stOils = document.getElementById('stOils').value;
  //  stNuts = document.getElementById('stNuts').value;
  // stCondiments = document.getElementById('stCondiments').value;
  // stDelCond = document.getElementById('stDelCond').value;
  // stKnockDown = document.getElementById('stKnockDown').value;

  if (stName.length < 1) {
    Swal.fire({
      icon: "error",
      title: "Oops...",
      text: "Please enter a store name.",
    });
    return;
  }
  if (stPlot.length < 1) {
    Swal.fire({
      icon: "error",
      title: "Oops...",
      text: "Please enter a Plot.",
    });
    return;
  }
  if (stFName.length < 1) {
    Swal.fire({
      icon: "error",
      title: "Oops...",
      text: "Please enter a first name.",
    });
    return;
  }
  if (stLName.length < 1) {
    Swal.fire({
      icon: "error",
      title: "Oops...",
      text: "Please choose a last name.",
    });
    return;
  }
  console.log("lat", lat);
  console.log("branchAddress", branchAddress);
  if (lat === null || typeof branchAddress === "undefined") {
    Swal.fire({
      icon: "error",
      title: "Oops...",
      text: "Please choose a store address.",
    });
    return;
  }
  if (!phonenumberBst(stPhone)) {
    Swal.fire({
      icon: "error",
      title: "Oops...",
      text: "Please enter a valid phone number",
    });
    return;
  }
  if (!validateEmail(stEmail)) {
    Swal.fire({
      icon: "error",
      title: "Oops...",
      text: "Please enter a valid email address",
    });
    return;
  }
  if (cuisineTypeArray.length === 0) {
    Swal.fire({
      icon: "error",
      title: "Oops...",
      text: "Please enter cuisine type.",
    });

    return;
  }

  if (stOpenTime === null) {
    Swal.fire({
      icon: "error",
      title: "Oops...",
      text: "Please enter opening time.",
    });

    return;
  }
  if (stCloseTime.length === null) {
    Swal.fire({
      icon: "error",
      title: "Oops...",
      text: "Please enter closing time.",
    });

    return;
  }

  let statusCode;
  let state;
  if (document.getElementById("radio1").checked) {
    statusCode = document.getElementById("radio1").value;
    state = "Published";
  } else if (document.getElementById("radio2").checked) {
    statusCode = document.getElementById("radio2").value;
    state = "Draft";
  }

  if (downloadUrl == " ") {
    Swal.fire({
      icon: "error",
      title: "Oops...",
      text: "Please add an image.",
    });
    return;
  }

  $("#stAddRest")
    .html(
      '<span class="spinner-border spinner-border-sm mr-2" role="status" aria-hidden="true"></span>Loading...'
    )
    .attr("disabled", true);

  document.getElementById("stAddRest").disabled = true;

  return $.ajax({
    url: "https://us-central1-dotdelivery-fbd89.cloudfunctions.net/searchBranches",
    data: JSON.stringify({
      branch: stName,
    }),
    processData: false,
    contentType: "application/json",
    type: "POST",
    success: function (data) {
      console.log(data);

      let exists = 0;
      if (data.length >= 1) {
        for (let i = 0; i < data.length; i++) {
          if (data.data[i]._highlightResult.branch.fullyHighlighted) {
            exists += 1;
          }
        }
        console.log("It exists: " + exists);

        if (exists > 0) {
          $("#stAddRest")
            .html(
              '<button type="button" class="btn btn-primary" id="stAddRest" onclick="addRest()">Save</button>\n'
            )
            .attr("disabled", false);
          Swal.fire({
            icon: "error",
            title: "Oops...",
            text: "A Branch with the same name already exists. It was either created by you or another account. If this is your branch, Please contact Dot Delivery Team.",
          }).then((result) => {});
        } else {
          return $.ajax({
            url: "https://us-central1-dotdelivery-fbd89.cloudfunctions.net/createManager",
            data: JSON.stringify({
              email: stEmail,
              photoUrl:
                "https://us-central1-dotdelivery-fbd89.cloudfunctions.net",
              name: stFName + " " + stLName,
            }),
            processData: false,
            contentType: "application/json",
            type: "POST",
            success: function (res) {
              if (res.status === 0) {
                $("#stAddRest")
                  .html(
                    '<button type="button" class="btn btn-primary" id="stAddRest" onclick="addRest()">Save</button>\n'
                  )
                  .attr("disabled", false);
                Swal.fire({
                  icon: "error",
                  title: "Oops...",
                  text: res.data.message,
                });
              } else if (res.status === 1) {
                let branchId = res.data.uid;
                return db
                  .collection("branches")
                  .doc(branchId)
                  .set({
                    status: state,
                    branchId: branchId,
                    restId: restId,
                    branch: stName,
                    branchFloor: stFloor,
                    branchPlot: stPlot,
                    cuisineTypeIds: cuisineTypeIds,
                    cuisineTypeArray: cuisineTypeArray,
                    branchAddress: branchAddress,
                    statusCode: statusCode,
                    restaurant: restName,
                    managerFirstName: stFName,
                    managerLastName: stLName,
                    managerFullName: stFName + " " + stLName,
                    branchEmail: stEmail,
                    branchPhone: stPhone,
                    latitude: lat,
                    longitude: lng,
                    openingTime: stOpenTime,
                    closingTime: stCloseTime,
                    createdByName: traderName,
                    createdBy: uid,
                    createdByEmail: email,
                    branchImg: downloadUrl,
                    timestamp: firebase.firestore.FieldValue.serverTimestamp(),
                  })
                  .then(function () {
                    console.log("Document written ");

                    return $.ajax({
                      url: "https://us-central1-dotdelivery-fbd89.cloudfunctions.net/exportBranches",

                      processData: false,
                      contentType: "application/json",
                      type: "POST",
                      success: function (resp) {
                        console.log("Algolia: " + JSON.stringify(resp));
                        Swal.fire({
                          icon: "success",
                          title: "Success",
                          text:
                            "You have successfully created a new branch of " +
                            restName,
                        }).then((result) => {
                          window.location.href = "view_branches.html";
                        });
                      },
                    });
                  })
                  .catch(function (error) {
                    $("#stAddRest")
                      .html(
                        '<button type="button" class="btn btn-primary" id="stAddRest" onclick="addRest()">Save</button>\n'
                      )
                      .attr("disabled", false);
                    console.error("Error adding document: ", error);
                    Swal.fire({
                      icon: "error",
                      title: "Oops...",
                      text: error,
                    }).then((result) => {});
                  });
              }
            },
          });

          // Add a new document with a generated id.
        }
      } else {
        return $.ajax({
          url: "https://us-central1-dotdelivery-fbd89.cloudfunctions.net/createManager",
          data: JSON.stringify({
            email: stEmail,
            photoUrl:
              "https://us-central1-dotdelivery-fbd89.cloudfunctions.net",
            name: stFName + " " + stLName,
          }),
          processData: false,
          contentType: "application/json",
          type: "POST",
          success: function (res) {
            // Add a new document with a generated id.
            if (res.status === 0) {
                $("#stAddRest")
                  .html(
                    '<button type="button" class="btn btn-primary" id="stAddRest" onclick="addRest()">Save</button>\n'
                  )
                  .attr("disabled", false);
                Swal.fire({
                  icon: "error",
                  title: "Oops...",
                  text: res.data.message,
                });
              } else if (res.status === 1) {
                let branchId = res.data.uid;
            return db
              .collection("branches")
              .doc(branchId)
              .set({
                status: state,
                branchId: branchId,
                restId: restId,
                branch: stName,
                branchFloor: stFloor,
                branchPlot: stPlot,
                cuisineTypeIds: cuisineTypeIds,
                cuisineTypeArray: cuisineTypeArray,
                branchAddress: branchAddress,
                statusCode: statusCode,
                restaurant: restName,
                managerFirstName: stFName,
                managerLastName: stLName,
                managerFullName: stFName + " " + stLName,
                branchEmail: stEmail,
                branchPhone: stPhone,
                latitude: lat,
                longitude: lng,
                openingTime: stOpenTime,
                closingTime: stCloseTime,
                createdByName: traderName,
                createdBy: uid,
                createdByEmail: email,
                branchImg: downloadUrl,
                timestamp: firebase.firestore.FieldValue.serverTimestamp(),
              })
              .then(function () {
                console.log("Document written ");

                return $.ajax({
                    url: "https://us-central1-dotdelivery-fbd89.cloudfunctions.net/exportBranches",

                    processData: false,
                    
                    type: "POST",
                    success: function (resp) {
                      console.log("Algolia: " + JSON.stringify(resp));
                      Swal.fire({
                        icon: "success",
                        title: "Success",
                        text:
                          "You have successfully created a new branch of " +
                          restName,
                      }).then((result) => {
                        window.location.href = "view_branches.html";
                      });
                    },
                  });
              })
              .catch(function (error) {
                $("#stAddRest")
                  .html(
                    '<button type="button" class="btn btn-primary" id="stAddRest" onclick="addRest()">Save</button>\n'
                  )
                  .attr("disabled", false);
                console.error("Error adding document: ", error);
                Swal.fire({
                  icon: "error",
                  title: "Oops...",
                  text: error,
                }).then((result) => {});
              });
            }
          },
        });
      }
    },
  });
}

function phonenumberBst(inputtxt) {
  var phoneno = /^\+?([0-9]{11})$/;
  if (inputtxt.match(phoneno)) {
    return true;
  } else {
    return phonenumberKe(inputtxt);
  }
}
function phonenumberKe(inputtxt) {
  var phoneno = /^\+?([0-9]{12})$/;
  if (inputtxt.match(phoneno)) {
    return true;
  } else {
    return false;
  }
}

function validateEmail(email) {
  const re =
    /^(([^<>()[\]\\.,;:\s@"]+(\.[^<>()[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;
  return re.test(String(email).toLowerCase());
}

function myMap() {}
