/**
 * initApp handles setting up UI event listeners and registering Firebase auth listeners:
 *  - firebase.auth().onAuthStateChanged: This listener is called when the user is signed in or
 *    out, and that is where we update the UI.
 */

let email;
let uid;
let db = firebase.firestore();
let restName;
let restId;
let traderName;
let traderId;
let menuImage;
let updateId;
let downloadUrl = " ";
let statusCodeCollect = "0";
let stateCollect = false;
let mealList;

function initApp() {
  // Listening for auth state changes.
  // [START authstatelistener]
  firebase.auth().onAuthStateChanged(function (user) {
    if (user) {
      // User is signed in.

      email = user.email;
      let emailVerified = user.emailVerified;
      uid = user.uid;
      let providerData = user.providerData;

      // console.log(email)

      let docRef = db.collection("traders").doc(uid);

      docRef
        .get()
        .then(function (doc) {
          if (doc.exists) {
            restName = doc.data().restaurant;
            restId = doc.data().restId;
            traderId = doc.data().traderId;
            traderName = doc.data().traderFullName;
            document.getElementById("proName").innerText =
              traderName + " (" + restName + ")";
            db.collection("menus")
              .where("restId", "==", restId)
              .orderBy("timestamp", "desc")
              .onSnapshot((snapshot) => {
                document.getElementById("menu_row").innerHTML = "";
                snapshot.forEach(function (docs) {
                  let menuName = docs.data().menuName;
                  let menuArray = docs.data().mealArray;
                  let mealArrayIds = docs.data().mealArrayIds;

                  let menuCategory = docs.data().menuCategories;
                  console.log("menu image " + docs.data().mealImage);
                  let menuImg;
                  if (docs.data().mealImage != " ") {
                    menuImg = docs.data().mealImage;
                  } else {
                    menuImg = "../images/noimage.png";
                  }
                  var div = document.getElementById("menu_row");

                  div.innerHTML +=
                    '<div class="col-xxxl-2 col-xl-3 col-lg-3 col-6">' +
                    '<div class="box overflow-h">' +
                    ' <div class="box-body p-0">' +
                    " <div>" +
                    '  <img class="rounded img-fluid" src=' +
                    menuImg +
                    ' alt="">' +
                    "   </div>" +
                    "  </div>" +
                    '<div class="box-body">' +
                    ' <div class="info-content">' +
                    ' <h5 class="mb-5 mt-0">' +
                    menuName +
                    "</h5>" +
                    ' <div class="d-flex justify-content-between align-items-center">' +
                    '  <h6 class="mb-0 text-primary">' +
                    menuCategory +
                    "</h6>" +
                    ' <div class="d-flex align-items-center">	' +
                    ' <span class="fs-18 mx-10 text-primary icon-Burger"><span class="path1"></span><span class="path2"></span><span class="path3"></span></span>' +
                    '   <h6 class="text-black mb-0">' +
                    menuArray.length +
                    " Meal(s)</h6>" +
                    "   </div>" +
                    "  </div>" +
                    "  </div>" +
                    "  </div>" +
                    '<div class="act-btn d-flex justify-content-between">' +
                    ' <div class="text-center mx-5">' +
                    '<a href="#"id=' +
                    docs.id +
                    ' data-bs-toggle="modal" data-bs-target="#modal-default" onclick ="editMenu(this.id)" class="waves-effect waves-circle btn btn-circle btn-success-light btn-xs mb-5"><i class="fa fa-eye-slash"></i></a>' +
                    '<small class="d-block">View/Edit</small>' +
                    "</div>" +
                    '<div class="text-center mx-5">' +
                    '<a href="#" id=' +
                    docs.id +
                    ' onclick ="deleteMenu(this.id)" class="waves-effect waves-circle btn btn-circle btn-primary-light btn-xs mb-5"><i class="fa fa-trash"></i></a>' +
                    ' <small class="d-block">Delete</small>' +
                    "</div>" +
                    "</div>" +
                    "</div>" +
                    " </div>" +
                    "  </div>";
                });
              });
            var div = document.getElementById("chooseMeal");
            db.collection("meals")
              .where("restId", "==", restId)
              .orderBy("timestamp", "desc")
              .onSnapshot((snapshot) => {
                mealList = snapshot;
                snapshot.forEach(function (docs) {
                  let mealId = docs.id;

                  let mealName = docs.data().mealName;
                  let mealDesc = docs.data().mealDesc;
                  let mealPrice = docs.data().mealPrice;
                  let mealCategory = docs.data().restCategories;

                  div.innerHTML +=
                    "<option value=" + mealId + ">" + mealName + "</option>";
                });
              });
          } else {
            Swal.fire({
              icon: "error",
              title: "Oops...",
              text: "You don not have access to this platform.",
            });
            toggleSignOut();
            // doc.data() will be undefined in this case
            console.log("No such document!");
          }
        })
        .catch(function (error) {
          console.log("Error getting document:", error);
        });
    } else {
      // User is signed out.
      console.log("User is signed out");
      window.location.href = "index.html";
    }
  });
  // [END authstatelistener]
}

window.onload = function () {
  initApp();
};

function toggleSignOut() {
  // [START signout]
  firebase.auth().signOut();
  // [END signout]
}

$(document).on("keydown", function (e) {
  var keyCode = e.which || e.keyCode;
  if (keyCode == 13) {
    // enter key code
    toggleSignIn();
  }
});

function editMenu(id) {
  console.log("Menu Id: " + id);
  updateId = id;
  db.collection("menus")
    .doc(id)
    .onSnapshot((doc) => {
      console.log("Current data: ", doc.data());

      menuImage = doc.data().mealImage;
      document.getElementById("menuName").value = doc.data().menuName;
      document.getElementById("imgPreview").src = doc.data().mealImage;
      $("#mealCategory").val(null).trigger("change");

      for (let i = 0; i < doc.data().menuCategories.length; i++) {
        var option = new Option(
          doc.data().menuCategories[i],
          doc.data().menuCategoriesIds[i],
          true,
          true
        );
        $("#mealCategory").append(option).trigger("change");
      }

      $("#chooseMeal").val(null).trigger("change");
      let selectedMeals = [];
      for (let i = 0; i < doc.data().mealArray.length; i++) {
        // var option = new Option(doc.data().mealArray[i], doc.data().mealArrayIds[i], true, true);
        // $("#chooseMeal").append(option).trigger('change');
        selectedMeals.push(doc.data().mealArrayIds[i]);
      }
      $("#chooseMeal").val(selectedMeals).trigger("change");
      if (doc.data().isCollectionOnly === false) {
        document.getElementById("noCollect").checked = true;
      } else {
        document.getElementById("yesCollect").checked = true;
      }
      var checkBox = document.getElementById("checkAvailable");

      if (doc.data().menuAvailability == true) {
        checkBox.checked = true;        
        document.getElementById("startTime").value = chunk(doc.data().startTime.toString(),2).join(':');
        document.getElementById("endTime").value = chunk(doc.data().endTime.toString(),2).join(':');
        openAvailability();
      } else {
        checkBox.checked = false;
        openAvailability();
      }

      var bs_modal = $("#modal");
      var image = document.getElementById("image");
      var cropper, reader, file;
      var maxwidth = 1024;
      var maxheight = 500;

      var finalAspectRatio = 1024 / 500;

      var imgwidth = 0;
      var imgheight = 0;
      downloadUrl = doc.data().mealImage;
      $("#modal").modal({
        backdrop: "static",
        keyboard: false,
      });

      $("body").on("change", ".image", function (e) {
        var files = e.target.files;
        var done = function (url) {
          image.src = url;

          image.onload = function () {
            imgwidth = this.width;
            imgheight = this.height;
            if (imgwidth === maxwidth && imgheight === maxheight) {
              console.log("Image size is " + maxwidth + "X" + maxheight);
              bs_modal.modal("show");
            } else {
              Swal.fire({
                icon: "error",
                title: "Oops...",
                text:
                  "Image size must be " + maxwidth + "w by " + maxheight + "h",
              });
            }
          };
        };

        if (files && files.length > 0) {
          file = files[0];

          if (URL) {
            done(URL.createObjectURL(file));
          } else if (FileReader) {
            reader = new FileReader();
            reader.onload = function (e) {
              done(reader.result);
            };
            reader.readAsDataURL(file);
          }
        }
      });

      bs_modal
        .on("shown.bs.modal", function () {
          cropper = new Cropper(image, {
            aspectRatio: finalAspectRatio,
            viewMode: 1,
            preview: ".preview",
          });
        })
        .on("hidden.bs.modal", function () {
          cropper.destroy();
          cropper = null;
        });

      $("#crop").click(function () {
        document.getElementById("crop").disabled = true;

        canvas = cropper.getCroppedCanvas({
          imageSmoothingEnabled: true,
          imageSmoothingQuality: "high",
        });

        canvas.toBlob(function (blob) {
          url = URL.createObjectURL(blob);
          let imPreview = document.getElementById("imgPreview");

          var reader = new FileReader();
          reader.readAsDataURL(blob);
          reader.onloadend = function () {
            var base64data = reader.result;
            console.log(base64data);
            // Get a reference to the storage service, which is used to create references in your storage bucket
            const storage = firebase.storage();

            // Create a storage reference from our storage service
            const storageRef = storage.ref();
            storageRef
              .child("menus/" + generateUUID())
              .putString(base64data, "data_url")
              .then((snapshot) => {
                snapshot.ref
                  .getDownloadURL()
                  .then(function (downloadURL) {
                    console.log("File available at", downloadURL);
                    downloadUrl = downloadURL;
                    imPreview.src = downloadURL;
                    document.getElementById("crop").disabled = false;

                    bs_modal.modal("hide");
                  })
                  .catch(function (error) {
                    document.getElementById("crop").disabled = false;

                    console.log("Error getting document:", error);
                  });
                console.log("Uploaded a data_url string!");
              });
          };
        });
      });
    });
}

function generateUUID() {
  // Public Domain/MIT
  var d = new Date().getTime(); //Timestamp
  var d2 = (performance && performance.now && performance.now() * 1000) || 0; //Time in microseconds since page-load or 0 if unsupported
  return "xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx".replace(/[xy]/g, function (c) {
    var r = Math.random() * 16; //random number between 0 and 16
    if (d > 0) {
      //Use timestamp until depleted
      r = (d + r) % 16 | 0;
      d = Math.floor(d / 16);
    } else {
      //Use microseconds since page-load if supported
      r = (d2 + r) % 16 | 0;
      d2 = Math.floor(d2 / 16);
    }
    return (c === "x" ? r : (r & 0x3) | 0x8).toString(16);
  });
}

function updateMenu() {
  $("#stUpdateMenu")
    .html(
      '<span class="spinner-border spinner-border-sm mr-2" role="status" aria-hidden="true"></span>Loading...'
    )
    .attr("disabled", true);

  document.getElementById("stUpdateMenu").disabled = true;

  let menuName = document.getElementById("menuName").value;

  let stCategoryIds = $("#mealCategory").select2("val");
  let stCategory = $("#mealCategory").select2("data");
  let categoryArray = [];
  for (let j = 0; j < stCategory.length; j++) {
    categoryArray.push(stCategory[j].text);
  }

  let stMealIds = $("#chooseMeal").select2("val");
  let stMeals = $("#chooseMeal").select2("data");
  let mealArray = [];
  for (let j = 0; j < stMeals.length; j++) {
    mealArray.push(stMeals[j].text);
  }

  let startTime;
  let endTime;
  var checkBox = document.getElementById("checkAvailable");
  if (checkBox.checked == true) {
    startTime = document.getElementById("startTime").value;
    endTime = document.getElementById("endTime").value;
    if (startTime.length !== 5) {
      Swal.fire({
        icon: "error",
        title: "Oops...",
        text: "Please enter a valid menu start time.",
      });
      return;
    }
    if (endTime.length !== 5) {
      Swal.fire({
        icon: "error",
        title: "Oops...",
        text: "Please enter a valid menu end time.",
      });
      return;
    }
    startTime = startTime.replace(":", "");
    endTime = endTime.replace(":", "");
  } else {
    startTime = "0000";
    endTime = "2400";
  }

  if (menuName.length < 1) {
    document.getElementById("stUpdateMenu").disabled = false;
    $("#stUpdateMenu")
      .html(
        '<button type="button" class="btn btn-primary" id="stUpdateMenu" onclick="updateMenu()">Save</button>\n'
      )
      .attr("disabled", false);

    Swal.fire({
      icon: "error",
      title: "Oops...",
      text: "Please enter name of the menu.",
    });
    return;
  }

  if (categoryArray.length === 0) {
    document.getElementById("stUpdateMenu").disabled = false;
    $("#stUpdateMenu")
      .html(
        '<button type="button" class="btn btn-primary" id="stUpdateMenu" onclick="updateMenu()">Save</button>\n'
      )
      .attr("disabled", false);
    Swal.fire({
      icon: "error",
      title: "Oops...",
      text: "Please choose categories for you menu.",
    });
    return;
  }
  if (mealArray.length === 0) {
    document.getElementById("stUpdateMenu").disabled = false;
    $("#stUpdateMenu")
      .html(
        '<button type="button" class="btn btn-primary" id="stUpdateMenu" onclick="updateMenu()">Save</button>\n'
      )
      .attr("disabled", false);
    Swal.fire({
      icon: "error",
      title: "Oops...",
      text: "Please choose meal for you menu.",
    });
    return;
  }
  if (document.getElementById("noCollect").checked) {
    statusCodeCollect = document.getElementById("noCollect").value;
    stateCollect = false;
  } else if (document.getElementById("yesCollect").checked) {
    statusCodeCollect = document.getElementById("yesCollect").value;
    stateCollect = true;
  }

  return db
    .collection("menus")
    .doc(updateId)
    .update({
      isCollectionOnly: stateCollect,
      isCollectionOnlyCode: statusCodeCollect,
      menuName: menuName,
      menuCategories: categoryArray,
      menuCategoriesIds: stCategoryIds,
      mealArray: mealArray,
      mealArrayIds: stMealIds,
      mealImage: downloadUrl,
      menuAvailability: checkBox.checked,
      startTime: parseInt(startTime),
      endTime: parseInt(endTime),
      updatedOn: firebase.firestore.FieldValue.serverTimestamp(),
      updatedBy: uid,
      updatedByName: traderName,
      updatedByEmail: email,
    })
    .then(function () {
      console.log("Document updated ");
      for (let i = 0; i < stMealIds.length; i++) {
        console.log("Meal Id: " + stMealIds[i])
        db.collection("meals").doc(stMealIds[i]).update({
            menuIds: firebase.firestore.FieldValue.arrayUnion(updateId)

        });
    }
      $("#stUpdateMenu")
      .html(
        '<button type="button" class="btn btn-primary" id="stUpdateMenu" onclick="updateMenu()">Save</button>\n'
      )
      .attr("disabled", false);
      Swal.fire({
        icon: "success",
        title: "Success",
        text: "You have successfully updated menu.",
      }).then((result) => {
        $("#modal-default").modal("hide");
        // window.location.href = 'menu_list.html'
      });
    })
    .catch(function (error) {
      document.getElementById("stUpdateMenu").disabled = false;
      $("#stUpdateMenu")
        .html(
          '<button type="button" class="btn btn-primary" id="stUpdateMenu" onclick="updateMenu()">Save</button>\n'
        )
        .attr("disabled", false);
      Swal.fire({
        icon: "error",
        title: "Oops...",
        text: error,
      }).then((result) => {});
    });
}

$(document).ready(function () {
  $("#mealCategory").select2({
    tags: true,
  });
  $("#chooseMeal").select2({
    tags: true,
  });
});

function deleteMenu(id) {
  console.log("Menu Id: " + id);
  Swal.fire({
    icon: "info",
    title: "Confirmation",
    text: "Are you sure you want to delete this menu? This might affect ongoing orders",
    showCancelButton: true,
  }).then((result) => {
    if (result.isConfirmed) {
      console.log("Menu Id: " + id);
      db.collection("menus")
        .doc(id)
        .delete()
        .then(() => {
          console.log("Document successfully deleted!");
        })
        .catch((error) => {
          console.error("Error removing document: ", error);
          Swal.fire({
            icon: "error",
            title: "Oops... something went wrong, please try again later.",
            text: error,
          }).then((result) => {});
        });
    }
  });
}

function openAvailability() {
  // Get the checkbox
  var checkBox = document.getElementById("checkAvailable");
  // Get the output text
  var availableDiv = document.getElementById("availableDiv");

  // If the checkbox is checked, display the output text
  if (checkBox.checked == true) {
    availableDiv.style.display = "block";
  } else {
    availableDiv.style.display = "none";
  }
}

function chunk(str, n) {
    var ret = [];
    var i;
    var len;

    for(i = 0, len = str.length; i < len; i += n) {
       ret.push(str.substr(i, n))
    }

    return ret
};
