/**
 * initApp handles setting up UI event listeners and registering Firebase auth listeners:
 *  - firebase.auth().onAuthStateChanged: This listener is called when the user is signed in or
 *    out, and that is where we update the UI.
 */

let email;
let uid;
let restName;
let traderName;
let restId;
let traderId;


var GET = {};
var query = window.location.search.substring(1).split("&");
for (var i = 0, max = query.length; i < max; i++) {
    if (query[i] === "") // check for trailing & with no param
        continue;

    var param = query[i].split("=");
    GET[decodeURIComponent(param[0])] = decodeURIComponent(param[1] || "");
}
let shopId = GET.shopId;
console.log("Shop Id: " + shopId);
let stName;
let stAddress;
let stEmail;
let stPhone;
let manName;
let manFname;
let manLName;
let cuisineArray;
let shopTypeArray;
let cuisineArrayIds;
let shopTypeArrayIds;
let stFloor;
let stPlot;
let stImg;
let stOpenTime;
let stCloseTime;
let placeSearch;
let autocomplete;
let lat;
let lng;
let restAddress;

let db = firebase.firestore();

var bs_modal = $('#modal');
var image = document.getElementById('image');
var cropper, reader, file;
var maxwidth = 512;
var maxheight = 512;

var finalAspectRatio = 1;

var imgwidth = 0;
var imgheight = 0;



$("body").on("change", ".image", function (e) {
    var files = e.target.files;
    var done = function (url) {
        image.src = url;

        image.onload = function () {
            imgwidth = this.width;
            imgheight = this.height;
            if (imgwidth === maxwidth && imgheight === maxheight) {
                console.log("Image size is " + maxwidth + "X" + maxheight)
                bs_modal.modal('show');
            } else {

                Swal.fire({
                    icon: 'error',
                    title: 'Oops...',
                    text: 'Image size must be ' + maxwidth + 'w by ' + maxheight + 'h'

                })

            }
        }


    };


    if (files && files.length > 0) {
        file = files[0];

        if (URL) {
            done(URL.createObjectURL(file));
        } else if (FileReader) {
            reader = new FileReader();
            reader.onload = function (e) {
                done(reader.result);
            };
            reader.readAsDataURL(file);
        }
    }
});

bs_modal.on('shown.bs.modal', function () {
    cropper = new Cropper(image, {
        aspectRatio: finalAspectRatio,
        viewMode: 1,
        preview: '.preview'
    });
}).on('hidden.bs.modal', function () {
    cropper.destroy();
    cropper = null;
});

$("#crop").click(function () {
    document.getElementById('crop').disabled = true;

    canvas = cropper.getCroppedCanvas({
        imageSmoothingEnabled: true,
        imageSmoothingQuality: 'high',
    });

    canvas.toBlob(function (blob) {
        url = URL.createObjectURL(blob);
        let imPreview = document.getElementById("imgPreview");



        var reader = new FileReader();
        reader.readAsDataURL(blob);
        reader.onloadend = function () {
            var base64data = reader.result;
            console.log(base64data);
            // Get a reference to the storage service, which is used to create references in your storage bucket
            const storage = firebase.storage();

            // Create a storage reference from our storage service
            const storageRef = storage.ref();
            storageRef.child('branches/' + generateUUID()).putString(base64data, 'data_url').then((snapshot) => {
                snapshot.ref.getDownloadURL().then(function (downloadURL) {
                    console.log('File available at', downloadURL);
                    stImg = downloadURL;
                    imPreview.src = downloadURL;
                    document.getElementById('crop').disabled = false;

                    bs_modal.modal('hide');

                }).catch(function (error) {
                    document.getElementById('crop').disabled = false;

                    console.log("Error getting document:", error);
                });
                console.log('Uploaded a data_url string!');
            });



        };
    });
});


function generateUUID() { // Public Domain/MIT
    var d = new Date().getTime();//Timestamp
    var d2 = (performance && performance.now && (performance.now() * 1000)) || 0;//Time in microseconds since page-load or 0 if unsupported
    return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, function (c) {
        var r = Math.random() * 16;//random number between 0 and 16
        if (d > 0) {//Use timestamp until depleted
            r = (d + r) % 16 | 0;
            d = Math.floor(d / 16);
        } else {//Use microseconds since page-load if supported
            r = (d2 + r) % 16 | 0;
            d2 = Math.floor(d2 / 16);
        }
        return (c === 'x' ? r : (r & 0x3 | 0x8)).toString(16);
    });
}

function initApp() {
    // Listening for auth state changes.
    // [START authstatelistener]
    firebase.auth().onAuthStateChanged(function (user) {
        if (user) {
            // User is signed in.



            email = user.email;
            let emailVerified = user.emailVerified;
            uid = user.uid;
            let providerData = user.providerData;

            console.log(user.metadata.lastSignInTime);
            let db = firebase.firestore();


            let docRef = db.collection("traders").doc(uid);

            docRef.get().then(function (doc) {
                if (doc.exists) {

                   
                    restName = doc.data().restaurant;
                    restId = doc.data().restId;
                    traderId = doc.data().traderId;
                    traderName = doc.data().traderFullName;
                    document.getElementById('proName').innerText = traderName + " (" + restName + ")";
                    db.collection("branches").doc(shopId).get().then(function (snapshot) {


                        stName = snapshot.data().branch;
                        stAddress = snapshot.data().branchAddress;
                        stEmail = snapshot.data().branchEmail;
                        stPlot = snapshot.data().branchPlot;
                        stFloor = snapshot.data().branchFloor;
                        stPhone = snapshot.data().branchPhone;
                        manName = snapshot.data().managerFullName;
                        manFName = snapshot.data().managerFirstName;
                        manLName = snapshot.data().managerLastName;
                        cuisineArray = snapshot.data().cuisineTypeArray;

                        cuisineArrayIds = snapshot.data().cuisineTypeIds;

                        stImg = snapshot.data().branchImg;
                        stOpenTime = snapshot.data().openingTime;
                        stCloseTime = snapshot.data().closingTime;
                        lat = snapshot.data().latitude;
                        lng = snapshot.data().longitude;
                        branchAddress = snapshot.data().branchAddress;

                        var mapProp = {
                            center: new google.maps.LatLng(lat, lng),
                            zoom: 18,
                            mapTypeId: google.maps.MapTypeId.ROADMAP,
                        };
                        var map = new google.maps.Map(document.getElementById("googleMap"), mapProp);
                        var marker = new google.maps.Marker({
                            position: new google.maps.LatLng(lat, lng),

                        });

                        marker.setMap(map);


                        document.getElementById('shop_name').innerText = "Branch: " + stName;
                        document.getElementById('shop_address').innerText = "Branch Address: " + stAddress;
                        document.getElementById('shop_manager').innerText = "Manager Name: " + manName;
                        document.getElementById('shop_phone').innerText = "Branch Phone: " + stPhone;
                        document.getElementById('shop_email').innerText = "Branch Email: " + stEmail;
                        document.getElementById('shop_image').src = stImg;
                        document.getElementById('shop_cuisines').innerText = cuisineArray;
                        document.getElementById('shop_cuisine').innerText = "Cuisine types: " + cuisineArray;
                        document.getElementById('open_close').innerText = "Opening Time: " + stOpenTime + " Closing Time: " + stCloseTime;
                        document.getElementById('stName').value = stName;
                        document.getElementById('storeAddress').value = stAddress;
                        document.getElementById('stOpenTime').value = stOpenTime;
                        document.getElementById('stCloseTime').value = stCloseTime;
                        document.getElementById('stFName').value = manFName;
                        document.getElementById('stLName').value = manLName;
                        document.getElementById('stEmail').value = stEmail;
                        document.getElementById('stPhone').value = stPhone;
                        document.getElementById('stFloor').value = stFloor;
                        document.getElementById('stPlot').value = stPlot;
                        document.getElementById('imgPreview').src = stImg;

                        $('#cuisineType').val(cuisineArrayIds);
                        $('#cuisineType').trigger("change");

                        if (snapshot.data().statusCode === "1") {
                            document.getElementById('radio1').checked = true;

                            document.getElementById("enableShop").style.display = "none";
                        } else {
                            document.getElementById('radio2').checked = true;

                            document.getElementById("disableShop").style.display = "none";

                        }

                    }).catch(function (error) {
                        console.log("Error getting document:", error);
                    });

                }


            }).catch(function (error) {
                console.log("Error getting document:", error);
            });



        } else {
            // User is signed out.
            console.log("User is signed out");
            window.location.href = 'index.html'

        }

    });
    // [END authstatelistener]

}

window.onload = function () {
    initApp();
};



function toggleSignOut() {

    // [START signout]
    firebase.auth().signOut();
    // [END signout]

}

$(document).on("keydown", function (e) {

    var keyCode = e.which || e.keyCode;
    if (keyCode == 13) // enter key code
    {
        toggleSignIn();
    }

});
function fillInAddress() {
    // Get the place details from the autocomplete object.
    const place = autocomplete.getPlace();
    console.log(place);

    lat = place.geometry.location.lat();
    // get lng
    lng = place.geometry.location.lng();
    branchAddress = place.name;

    console.log(lat + " " + lng + " " + branchAddress);


    var mapProp = {
        center: new google.maps.LatLng(lat, lng),
        zoom: 18,
        mapTypeId: google.maps.MapTypeId.ROADMAP,
    };
    var map = new google.maps.Map(document.getElementById("googleMap"), mapProp);
    var marker = new google.maps.Marker({
        position: new google.maps.LatLng(lat, lng),

    });

    marker.setMap(map);
}
$(document).ready(function () {
    $("#restCat").select2();
    $("#cuisineType").select2();

});

function initMap() {
    const center = { lat: 50.064192, lng: -130.605469 };
    // Create a bounding box with sides ~10km away from the center point
    const defaultBounds = {
        north: center.lat + 0.1,
        south: center.lat - 0.1,
        east: center.lng + 0.1,
        west: center.lng - 0.1,
    };
    const input = document.getElementById("storeAddress");
    const options = {
        bounds: defaultBounds,

        fields: ["geometry", "name", "formatted_address"],
        origin: center,
        strictBounds: false,
        types: ["establishment"],
    };
    autocomplete = new google.maps.places.Autocomplete(input, options);
    autocomplete.addListener("place_changed", fillInAddress);
    var mapProp = {
        center: new google.maps.LatLng(-24.653257, 25.906792),
        zoom: 10,
        mapTypeId: google.maps.MapTypeId.ROADMAP,
    };
    var map = new google.maps.Map(document.getElementById("googleMap"), mapProp);
    $('#modal-default').on('shown.bs.modal', function () {
        google.maps.event.trigger(map, "resize");

        $('#modal').modal({
            backdrop: 'static',
            keyboard: false
        })

    });
}
function disableBranch() {
    console.log("Branch Id: " + shopId);
    Swal.fire({
        icon: 'info',
        title: 'Confirmation',
        text: 'Are you sure you want to disable this branch? This might affect ongoing orders',
        showCancelButton: true,

    }).then((result) => {
        if (result.isConfirmed) {

            db.collection("branches").doc(shopId).update({
                statusCode: "2",
                status: "Unpublished"

            }).then(() => {
                console.log("Document successfully disabled!");
                Swal.fire({
                    icon: 'success',
                    title: 'Success',
                    text: 'Branch successfully disabled',

                }).then((result) => {
                    window.location = "view_branches.html"
                })
            }).catch((error) => {
                console.error("Error removing document: ", error);
                Swal.fire({
                    icon: 'error',
                    title: 'Oops... something went wrong, please try again later.',
                    text: error

                }).then((result) => {

                })
            });
        }

    })


}

function enableBranch() {
    console.log("Branch Id: " + shopId);
    Swal.fire({
        icon: 'info',
        title: 'Confirmation',
        text: 'Are you sure you want to enable this branch?',
        showCancelButton: true,

    }).then((result) => {
        if (result.isConfirmed) {

            db.collection("branches").doc(shopId).update({
                statusCode: "1",
                status: "Published"

            }).then(() => {
                Swal.fire({
                    icon: 'success',
                    title: 'Success',
                    text: 'Branch successfully enabled',

                }).then((result) => {
                    window.location = "view_branches.html"
                })
            }).catch((error) => {
                console.error("Error removing document: ", error);
                Swal.fire({
                    icon: 'error',
                    title: 'Oops... something went wrong, please try again later.',
                    text: error

                }).then((result) => {

                })
            });
        }

    })


}
function addRest() {


    let cuisineTypeIds = $('#cuisineType').select2("val");
    let cuisineType = $('#cuisineType').select2("data");
    let cuisineTypeArray = []
    for (let j = 0; j < cuisineType.length; j++) {
        cuisineTypeArray.push(cuisineType[j].text)
    }

    let stName = document.getElementById('stName').value;
    let stOpenTime = document.getElementById('stOpenTime').value;
    let stCloseTime = document.getElementById('stCloseTime').value;


    stFloor = document.getElementById('stFloor').value;
    let stPlot = document.getElementById('stPlot').value;

    // let status = document.getElementById('active').value;
    let stFName = document.getElementById('stFName').value;
    let stLName = document.getElementById('stLName').value;

    let stEmail = document.getElementById('stEmail').value;

    let stPhone = document.getElementById('stPhone').value;

    // stBaking = document.getElementById('stBaking').value;
    // stOils = document.getElementById('stOils').value;
    //  stNuts = document.getElementById('stNuts').value;
    // stCondiments = document.getElementById('stCondiments').value;
    // stDelCond = document.getElementById('stDelCond').value;
    // stKnockDown = document.getElementById('stKnockDown').value;






    if (stName.length < 1) {


        Swal.fire({
            icon: 'error',
            title: 'Oops...',
            text: 'Please enter a store name.'

        })
        return;
    }
    if (stPlot.length < 1) {


        Swal.fire({
            icon: 'error',
            title: 'Oops...',
            text: 'Please enter a Plot.'

        })
        return;
    }
    if (stFName.length < 1) {


        Swal.fire({
            icon: 'error',
            title: 'Oops...',
            text: 'Please enter a first name.'

        })
        return;
    }
    if (stLName.length < 1) {

        Swal.fire({
            icon: 'error',
            title: 'Oops...',
            text: 'Please choose a last name.'

        })
        return;
    }
    console.log("lat", lat);
    console.log("branchAddress", branchAddress);
    if (lat === null || typeof branchAddress === "undefined") {


        Swal.fire({
            icon: 'error',
            title: 'Oops...',
            text: 'Please choose a store address.'

        })
        return;
    }
    if (!phonenumberBst(stPhone)) {


        Swal.fire({
            icon: 'error',
            title: 'Oops...',
            text: 'Please enter a valid phone number'

        });
        return;
    }
    if (!validateEmail(stEmail)) {


        Swal.fire({
            icon: 'error',
            title: 'Oops...',
            text: 'Please enter a valid email address'

        });
        return;
    }
    if (cuisineTypeArray.length === 0) {

        Swal.fire({
            icon: 'error',
            title: 'Oops...',
            text: 'Please enter cuisine type.'

        })

        return;
    }


    if (stOpenTime === null) {

        Swal.fire({
            icon: 'error',
            title: 'Oops...',
            text: 'Please enter opening time.'

        })

        return;
    }
    if (stCloseTime.length === null) {

        Swal.fire({
            icon: 'error',
            title: 'Oops...',
            text: 'Please enter closing time.'

        })

        return;
    }



    let statusCode;
    let state;
    if (document.getElementById('radio1').checked) {
        statusCode = document.getElementById('radio1').value;
        state = "Published";

    } else if (document.getElementById('radio2').checked) {
        statusCode = document.getElementById('radio2').value;
        state = "Draft";
    }

    if (stImg == ' ') {

        Swal.fire({
            icon: 'error',
            title: 'Oops...',
            text: 'Please add an image.'
        })
        return;
    }

    $('#stAddRest').html('<span class="spinner-border spinner-border-sm mr-2" role="status" aria-hidden="true"></span>Loading...').attr('disabled', true);

    document.getElementById('stAddRest').disabled = true;

    return db.collection("branches").doc(shopId).update({
        status: state,
    
        branch: stName,
        branchFloor: stFloor,
        branchPlot: stPlot,
        cuisineTypeIds: cuisineTypeIds,
        cuisineTypeArray: cuisineTypeArray,
        branchAddress: branchAddress,
        statusCode: statusCode,
        managerFirstName: stFName,
        managerLastName: stLName,
        managerFullName: stFName + " " + stLName,
        branchEmail: stEmail,
        branchPhone: stPhone,
        latitude: lat,
        longitude: lng,
        openingTime: stOpenTime,
        closingTime: stCloseTime,
        updatedByName: traderName,
        updatedBy: uid,
        updatedByEmail: email,
        branchImg: stImg,
        updatedOn: firebase.firestore.FieldValue.serverTimestamp(),
    })
        .then(function () {
            console.log("Document written ");

            Swal.fire({
                icon: 'success',
                title: 'Success',
                text: 'You have successfully updated branch  '

            }).then((result) => {
                window.location.href = 'view_branches.html'
            })

        })
        .catch(function (error) {
            $('#stAddRest').html('<button type="button" class="btn btn-primary" id="stAddRest" onclick="addRest()">Save</button>\n').attr('disabled', false);
            console.error("Error adding document: ", error);
            Swal.fire({
                icon: 'error',
                title: 'Oops...',
                text: error

            }).then((result) => {

            })
        });

}
function phonenumberBst(inputtxt) {
    var phoneno = /^\+?([0-9]{11})$/;
    if ((inputtxt.match(phoneno))) {
        return true;
    }
    else {

        return phonenumberKe(inputtxt);
    }
}
function phonenumberKe(inputtxt) {
    var phoneno = /^\+?([0-9]{12})$/;
    if ((inputtxt.match(phoneno))) {
        return true;
    }
    else {

        return false;
    }
}

function validateEmail(email) {
    const re = /^(([^<>()[\]\\.,;:\s@"]+(\.[^<>()[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/;
    return re.test(String(email).toLowerCase());
}
